{$I 4i30bits}
const

RefRate = 500000; { the reference count rate when counter 3 is used as a timer }

type LongIntByteRec = record
  Lowbyte  : byte;
  MidLbyte : byte;
  MidHbyte : byte;
  Highbyte : byte;
end;

type LongIntWordRec = record
  LowWord  : word;
  HighWord : word;
end;

procedure Init4I30;
begin
  { first initialize our port addresses soe's we dont gotta do the add every time }
  Data0Port := I30BasePort + Data0PortOfs;
  Data1Port := I30BasePort + Data1PortOfs;
  Data2Port := I30BasePort + Data2PortOfs;
  Data3Port := I30BasePort + Data3PortOfs;
  CountEnPort := I30BasePort + CountEnPortOfs;
  StatusPort := I30BasePort + StatusPortOfs;
  IDXInPort := I30BasePort + IDXInPortOfs;
  CommandPort := I30BasePort + CommandPortOfs;
  port[CommandPort] := ClearAllCom;
  port[CountEnPort] := 0;
  port[CountEnPort] := AllChanEn;
end;

function SlowReadCounter(ch : byte): longint;
var
tdata : byte;
tlong : longint;
begin
  ch := ch and ChSelMask;  { drop any wayward bits }
  tdata := ReadCom or ch;  { or in the channel select bits }
  port[CommandPort] := tdata;
  while (port[StatusPort] and BusyMask) <> 0 do ; { wait for read to finish }
  { then read the latched data }
  LongIntByteRec(tlong).Lowbyte  := port[Data0Port];
  LongIntByteRec(tlong).MidLbyte := port[Data1Port];
  LongIntByteRec(tlong).MidHbyte := port[Data2Port];
  LongIntByteRec(tlong).Highbyte := port[Data3Port];
  SlowReadCounter := tlong;
end;

function ReadCounter(chan : byte): longint;
var
tlong : longint;
begin
  asm
    mov dx,CommandPort;
    mov al,chan
    and al,ChSelMask
    { drop any wayward bits }
    or al,ReadCom
    out dx,al
    { issue read command }
    @loop:
    in al,dx
    and al,BusyMask
    jnz @loop
    mov dx,Data0Port
    in ax,dx
    {get low word of 32 bit count}
    mov LongIntWordRec(tlong).LowWord,ax
    inc dx
    inc dx
    in ax,dx
    {get high word of 32 bit count}
    mov LongIntWordRec(tlong).HighWord,ax
  end;
  ReadCounter := tlong;
end;

procedure ReadCounterAndDelta(chan : byte; var count: longint; var time:longint);
var tcount,ttime : longint;
begin
  asm
    { This can be much improved by someone who }
    { knows x86 assembler better than I }
    mov dx,CommandPort;
    mov al,chan
    or al,ReadCom
    cli
    { disable interrupts to prevent timing errors }
    out dx,al
    { issue read ch command }
    @loop:
    in al,dx
    and al,BusyMask
    jnz @loop
    mov dx,Data0Port
    in ax,dx
    mov LongIntWordRec(tcount).LowWord,ax
    inc dx
    inc dx
    in ax,dx
    mov LongIntWordRec(tcount).HighWord,ax
    mov dx,CommandPort;
    mov al,Read3Com
    out dx,al
    @loop2:
    in al,dx
    and al,BusyMask
    jnz @loop2
    mov dx,Data0Port
    in ax,dx
    mov LongIntWordRec(ttime).LowWord,ax
    inc dx
    inc dx
    in ax,dx
    mov LongIntWordRec(ttime).HighWord,ax
  end;
  time := ttime;
  count := tcount;
end;

procedure ClearCounter(ch : byte);
var
tdata : byte;
begin
  ch := ch and ChSelMask;   { drop any wayward bits }
  tdata := ClearCom or ch;  { or in the channel select bits }
  port[CommandPort] := tdata;
  while (port[StatusPort] and BusyMask) <> 0 do ; { wait for clear to finish }
end;
